﻿using System;
using System.Collections.Generic;
using Ozeki.VoIP;
using System.Timers;
using Ozeki.Media;

namespace _03_Voice_control_IVR
{
    class CallHandler
    {
        ICall _call;
        MediaConnector _mediaConnector;
        PhoneCallAudioSender _phoneCallAudioSender;
        Timer _greetingMessageTimer;
        PhoneCallAudioReceiver _phoneCallAudioReceiver;
        IEnumerable<string> _choices;
        SpeechToText _stt;
        TextToSpeech _tts;
        MP3StreamPlayback _mp3Player;

        public CallHandler(ICall call)
        {
            _greetingMessageTimer = new Timer();
            _greetingMessageTimer.Interval = 30000;
            _greetingMessageTimer.Elapsed += greetingMessageTimer_Elapsed;
            _call = call;
            _phoneCallAudioSender = new PhoneCallAudioSender();
            _phoneCallAudioSender.AttachToCall(call);
            _mediaConnector = new MediaConnector();
            _phoneCallAudioReceiver = new PhoneCallAudioReceiver();
            _phoneCallAudioReceiver.AttachToCall(call);
            _choices = new List<string>() { "first", "second"};
            _stt = SpeechToText.CreateInstance(_choices);
        }

        void greetingMessageTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            StartGreetingMessage();
        }

        private void StartGreetingMessage()
        {
            DisposeCurrentConnector();
            TextToSpeech("Introduce you the Interactive Voice Mail aka IVR example code written with Ozeki VoIP SDK. For more information about Ozeki Ltd. please say it first. If you say second, you can listen a sample mp3 file.");
        }

        public event EventHandler Completed;

        public void Start()
        {
            _mediaConnector.Connect(_phoneCallAudioReceiver, _stt);
            _call.CallStateChanged += call_CallStateChanged;
            _stt.WordHypothesized += CallHandler_WordHypothesized;
            _call.Answer();
        }

        void CallHandler_WordHypothesized(object sender, SpeechDetectionEventArgs e)
        {
            DisposeCurrentConnector();
            Console.WriteLine(e.Word);
            switch (e.Word)
            {
                case "first": TextToSpeech("Ozeki Informatics Ltd. is a leading mobile messaging software vendor. It offers easy-to-use and feature-rich telcommunication products for businesses and organizations."); break;
                case "second": MP3ToSpeaker(); break;
            }
        }

        void MP3ToSpeaker()
        {
            _greetingMessageTimer.Stop();

            _mp3Player = new MP3StreamPlayback("../../test.mp3");

            _mediaConnector.Connect(_mp3Player, _phoneCallAudioSender);
            _mp3Player.Stopped += mp3Player_Stopped;
            _mp3Player.Start();
        }

        void mp3Player_Stopped(object sender, EventArgs e)
        {
            _greetingMessageTimer.Start();
        }

        void call_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            if (e.State == CallState.Answered)
            {
                Console.WriteLine("Call answered!");
                _greetingMessageTimer.Start();
                StartGreetingMessage();
            }
        }

        void TextToSpeech(string text)
        {
            _greetingMessageTimer.Stop();

            _tts = new TextToSpeech();
            _tts.Stopped += tts_Stopped;

            _mediaConnector.Connect(_tts, _phoneCallAudioSender);
            _tts.AddAndStartText(text);
        }

        void tts_Stopped(object sender, EventArgs e)
        {
            _greetingMessageTimer.Start();
        }

        void DisposeCurrentConnector()
        {
            if (_tts != null)
            {
                _mediaConnector.Disconnect(_tts, _phoneCallAudioSender);
            }
            if (_mp3Player != null)
            {
                _mediaConnector.Disconnect(_mp3Player, _phoneCallAudioSender);
            }
        }
    }
}
